import sys
import os
import re
import configparser
import tempfile
import ast
from importlib.metadata import PackageNotFoundError

from asammdf import Signal
from asammdf.blocks.utils import TERMINATED
import asammdf



def read_export_options(options: configparser.ConfigParser, format, filepath):
    """
    Read export options from configuration file
    :param options: Export options obtained from ini file
    :type options: configparser.ConfigParser
    :param format: Export format (hdf5, mat, parquet, csv, asc)
    :type format: str
    :param filepath: Destination file path
    :type filepath: str
    """
    export_options = {}
    if format == "HDF5":
        export_options = read_csv_export_options(options, filepath)
    elif format == "MAT":
        export_options = read_mat_export_options(options, filepath)
    elif format == "Parquet":
        export_options = read_csv_export_options(options, filepath)
    elif format == "CSV":
        export_options = read_csv_export_options(options, filepath)
    elif format == "ASC":
        export_options = read_csv_export_options(options, filepath)
    return export_options


def read_csv_export_options(options: configparser.ConfigParser, filepath):
    """
    Read CSV export options from configuration file
    :param options: Export options obtained from ini file
    :type options: configparser.ConfigParser
    :param filepath: Destination file path
    :type filepath: str
    """	
    csv_export_options = {
        "fmt": "csv",
        "filename": filepath,
        "single_time_base": True if options.get('export', 'single_time_base') == "true" else False,
        "empty_channels": options.get('export', 'empty_channels'),
        "time_as_date": True if options.get('cut', 'time_as_date') == "true" else False,
	"time_from_zero": True if options.get('cut', 'time_from_zero') == "true" else False,
        "raw": True if options.get('export', 'raw_values') == "true" else False,
        "delimiter": chr(int((options.get('export', 'delimiter')))),
        "doublequote": True if options.get('export', 'doublequote') == "true" else False,
        "escapechar": chr(int((options.get('export', 'escapechar')))),
        "lineterminator": options.get('export', 'lineterminator').replace("\\r", "\r").replace("\\n", "\n"),     
        "quotechar": chr(int((options.get('export', 'quotechar')))),
        "quoting": options.get('export', 'quoting'),
        "add_units": True if options.get('export', 'add_units') == "true" else False,
        "use_display_names": True if options.get('export', 'use_display_names') == "true" else False,
    }
    return csv_export_options


def read_mat_export_options(options: configparser.ConfigParser, filepath):
    """
    Read MAT export options from configuration file
    :param options: Export options obtained from ini file
    :type options: configparser.ConfigParser
    :param filepath: Destination file path
    :type filepath: str
    """
    mat_export_options = {
        "fmt": "mat",
        "filename": filepath,
        "single_time_base": True if options.get('export', 'single_time_base') == "true" else False,
        "empty_channels": options.get('export', 'empty_channels'),
	"time_from_zero": True if options.get('cut', 'time_from_zero') == "true" else False,
        "format": options.get('export', 'mat_format'),
    }
    return mat_export_options


if __name__ == "__main__":

    print("Export data")

    temp_dir = tempfile.gettempdir()
    config_file_path = temp_dir + "/" + "export_options_" + sys.argv[1] + ".ini"
    print("--> config_file_path : " + config_file_path)
    if not os.path.exists(config_file_path):
        raise FileNotFoundError(f"Configuration file not found: {config_file_path}")

    config = configparser.ConfigParser()
    config.read(config_file_path)

    db_file_name = config.get('global', 'db_file_name')
    db_file_path = temp_dir + "/" + db_file_name
    print("--> db_file_path : " + db_file_path)

    #mdf_directory_path = config.get('global', 'directory_path')
    #mdf_file_name = config.get('global', 'file_name')
    #mdf_file_path = mdf_directory_path + "/" + mdf_file_name
    mdf_file_path = config.get('global', 'src_file_path')
    print("--> mdf_file_path : " + mdf_file_path)
    if not os.path.exists(config_file_path):
        raise FileNotFoundError(f"MDF file not found: {mdf_file_path}")

    output_format = config.get('export', 'output_format')
    print("--> output_format : " + output_format)

    if output_format == "HDF5":
        try:
            from h5py import File as HDF5  # noqa: F401
        except ImportError:
            raise PackageNotFoundError("h5py package not found")

    elif output_format == "MAT":
        if config.get('export', 'mat_format') == "7.3":
            try:
                from hdf5storage import savemat
            except ImportError:
                raise PackageNotFoundError("hdf5storage package not found")
        else:
            try:
                from scipy.io import savemat  # noqa: F401
            except ImportError:
                raise PackageNotFoundError("scipy package not found")

    elif output_format == "Parquet":
        try:
            from fastparquet import write as write_parquet  # noqa: F401
        except ImportError:
            raise PackageNotFoundError("fastparquet package not found")

    version = "4.10"

    if output_format == "MDF":
        version = config.get('export', 'version')
    if output_format == "HDF5":
        suffix = ".hdf"
    elif output_format == "MAT":
        suffix = ".mat"
    elif output_format == "Parquet":
        suffix = ".parquet"
    elif output_format == "CSV":
        suffix = ".csv"
    elif output_format == "ASC":
        suffix = ".asc"

    with asammdf.MDF(name=mdf_file_path) as mdf_file:
        split_size = int(config.get('export', 'data_block_size')) if output_format == "MDF" else 0
        mdf_file.configure(read_fragment_size=split_size)

        mdf = None

        integer_interpolation = int(config.get('resample', 'integer_interpolation'))
        float_interpolation = int(config.get('resample', 'float_interpolation'))

        needs_filter = True if config.get('filter', 'needs_filter') == "true" else False
        if needs_filter:

            channel_list_str = config.get('filter', 'channel_list')
            
            pattern = r'\[(.*?)\]'
            matches = re.findall(pattern, channel_list_str)

            channel_list = []
            for match in matches:
            
                elements = match.split(',')

                parsed_elements = []
                for el in elements:
                    el = el.strip()
                    if not el.isdigit():  # Si ce n'est pas un nombre, on l'entoure de guillemets
                        el = f'"{el}"'
                    parsed_elements.append(ast.literal_eval(el))  # ast.literal_eval pour convertir en bon type
        
                channel_list.append(parsed_elements)
        
            result = mdf_file.filter(
                channels=channel_list,
                version=version if output_format == "MDF" else "4.10",
            )

            if result is TERMINATED:
                sys.exit("Failed to filter data")
            else:
                mdf = result

            print("--> mdf_file.configure : ")
            print("----> read_fragment_size : " + str(split_size))
            print("----> write_fragment_size : " + str(split_size))
            print("----> integer_interpolation : " + str(integer_interpolation))
            print("----> float_interpolation : " + str(float_interpolation))

            mdf.configure(
                read_fragment_size=split_size,
                write_fragment_size=split_size,
                integer_interpolation=integer_interpolation,
                float_interpolation=float_interpolation,
            )

        needs_cut = True if config.get('cut', 'needs_cut') == "true" else False
        if needs_cut:

            cut_start = float(config.get('cut', 'cut_start'))
            cut_stop = float(config.get('cut', 'cut_stop'))
            whence = int(config.get('cut', 'whence'))
            time_from_zero = True if config.get('cut', 'time_from_zero') == "true" else False

            print("--> mdf_file.cut :")
            print("----> cut_start : " + str(cut_start))
            print("----> cut_stop : " + str(cut_stop))
            print("----> whence : " + str(whence))
            print("----> time_from_zero : " + str(time_from_zero))

            target = mdf_file.cut if mdf is None else mdf.cut
            result = target(start=cut_start,
                            stop=cut_stop,
                            whence=whence,
                            version=version if output_format == "MDF" else "4.10",
                            time_from_zero=time_from_zero)

            if result is TERMINATED:
                sys.exit("Failed to cut data")
            else:
                if mdf is None:
                    mdf = result
                else:
                    mdf.close()
                    mdf = result

            print("--> mdf_file.configure : ")
            print("----> read_fragment_size : " + str(split_size))
            print("----> write_fragment_size : " + str(split_size))
            print("----> integer_interpolation : " + str(integer_interpolation))
            print("----> float_interpolation : " + str(float_interpolation))

            mdf.configure(
                read_fragment_size=split_size,
                write_fragment_size=split_size,
                integer_interpolation=integer_interpolation,
                float_interpolation=float_interpolation,
            )

        needs_resample = True if config.get('resample', 'needs_resample') == "true" else False
        if needs_resample:

            print("--> mdf.resample : ")
            print("----> version : " + version)
            
            resample_strategy = int(config.get('resample', 'resample_strategy'))
            if resample_strategy == 2:
                raster_channel = config.get('resample', 'raster_channel')
                print("----> resample with timestamps of channel " + raster_channel)
                raster = raster_channel
            elif resample_strategy == 1:
                raster = float(config.get('resample', 'raster'))
                print("----> resample with frequency " + str(raster))
            else:
                sys.exit("Invalid resampling strategy")

            resample_time_from_zero = True if config.get('cut', 'time_from_zero') == "true" else False
            print("----> time_from_zero : " + str(resample_time_from_zero))

            target = mdf_file.resample if mdf is None else mdf.resample
            result = target(
                raster=raster,
                version=version if output_format == "MDF" else "4.10",
                time_from_zero=resample_time_from_zero,
            )

            if result is TERMINATED:
                sys.exit("Failed to resample data")
            else:
                if mdf is None:
                    mdf = result
                else:
                    mdf.close()
                    mdf = result

            print("--> mdf_file.configure : ")
            print("----> read_fragment_size : " + str(split_size))
            print("----> write_fragment_size : " + str(split_size))
            print("----> integer_interpolation : " + str(integer_interpolation))
            print("----> float_interpolation : " + str(float_interpolation))

            mdf.configure(
                read_fragment_size=split_size,
                write_fragment_size=split_size,
                integer_interpolation=integer_interpolation,
                float_interpolation=float_interpolation,
            )

        if output_format == "MDF":

            if mdf is None:

                print("--> mdf_file.convert : ")
                print("----> version : " + version)

                result = mdf_file.convert(
                    version=version,
                )

                if result is TERMINATED:
                    sys.exit("Failed to convert data")
                else:
                    mdf = result

            print("--> mdf_file.configure : ")
            print("----> read_fragment_size : " + str(split_size))
            print("----> write_fragment_size : " + str(split_size))
            print("----> integer_interpolation : " + str(integer_interpolation))
            print("----> float_interpolation : " + str(float_interpolation))

            mdf.configure(
                read_fragment_size=split_size,
                write_fragment_size=split_size,
                integer_interpolation=integer_interpolation,
                float_interpolation=float_interpolation,
            )

            if mdf.version != version:
                print("--> mdf_file.convert : ")
                print("----> version : " + version)

                result = mdf.convert(
                    version=version,
                )

                mdf.close()
                mdf = result

            if version >= "4.00":
                suffix = ".mf4"
            else:
                suffix = ".mdf"

            print("--> mdf_file.configure : ")
            print("----> read_fragment_size : " + str(split_size))
            print("----> write_fragment_size : " + str(split_size))
            print("----> integer_interpolation : " + str(integer_interpolation))
            print("----> float_interpolation : " + str(float_interpolation))

            mdf.configure(
                read_fragment_size=split_size,
                write_fragment_size=split_size,
                integer_interpolation=integer_interpolation,
                float_interpolation=float_interpolation,
            )

            if version >= "4.00":

                with open(db_file_path, 'rb') as db_file:

                    print("--> mdf.attach : ")
                    print("----> file_name : " + db_file_path)

                    binary_data = db_file.read()
                    index = mdf.attach(
                        data=binary_data,
                        file_name=db_file_name,
                        mime="application/octet-stream",
                        compression=False,
                        comment="Acquisition configuration file"
                    )

            #dst_file_name = config.get('export', 'dst_file_name')
            #dst_file_path = mdf_directory_path + "/" + dst_file_name + suffix
            dst_file_path = config.get('export', 'dst_file_path')

            print("--> mdf_file.save : ")
            print("----> dst : " + dst_file_path)

            result = mdf.save(
                dst=dst_file_path,
                overwrite=True,
            )

            if result is TERMINATED:
                sys.exit("Failed to convert data")

        else:

            #dst_file_name = config.get('export', 'dst_file_name')
            #dst_file_path = mdf_directory_path + "/" + dst_file_name + suffix
            dst_file_path = config.get('export', 'dst_file_path')

            kwargs = read_export_options(config, output_format, dst_file_path)

            target = mdf_file.export if mdf is None else mdf.export
            target(**kwargs)
